@@ -37,6 +37,12 @@ COMPAT_MAYBE = 2
 
 
 def composite_action(*functions):
+    """Makes an action that just calls several other actions in sequence.
+
+    Useful to implement ``myplugin setup`` in terms of ``myplugin setup/part1``
+    and ``myplugin setup/part2``: simply use
+    ``act1n2 = composite_action(act1, act2)``.
+    """
     def wrapper(args):
         for function in functions:
             function(args)
@@ -44,6 +50,8 @@ def composite_action(*functions):
 
 
 def target_must_exist(func):
+    """Decorator that checks that ``args.target`` exists.
+    """
     @functools.wraps(func)
     def wrapper(args):
         target = Path(args.target[0])
@@ -68,11 +76,15 @@ unique_names = unique_names()
 
 
 def make_unique_name(prefix):
+    """Makes a unique (random) bytestring name, starting with the given prefix.
+    """
     assert isinstance(prefix, bytes)
     return prefix + next(unique_names)
 
 
 def shell_escape(s):
+    """Given bl"a, returns "bl\\"a".
+    """
     if isinstance(s, bytes):
         s = s.decode('utf-8')
     if any(c in s for c in string.whitespace + '*$\\"\''):
@@ -84,6 +96,12 @@ def shell_escape(s):
 
 
 def load_config(pack):
+    """Utility method loading the YAML configuration from inside a pack file.
+
+    Decompresses the config.yml file from the tarball to a temporary file then
+    loads it. Note that decompressing a single file is inefficient, thus
+    calling this method can be slow.
+    """
     tmp = Path.tempdir(prefix='reprozip_')
     try:
         # Loads info from package
@@ -105,6 +123,8 @@ def load_config(pack):
 
 
 class AptInstaller(object):
+    """Installer for deb-based systems (Debian, Ubuntu).
+    """
     def __init__(self, binary):
         self.bin = binary
 
@@ -123,7 +143,7 @@ class AptInstaller(object):
             if status is not None:
                 required_pkgs.discard(pkg.name)
         if required_pkgs:
-            logging.error("Error: some packages could not be installed:%s" %
+            logging.error("Error: some packages could not be installed:%s",
                           ''.join("\n    %s" % pkg for pkg in required_pkgs))
 
         return r, pkgs_status
@@ -150,7 +170,7 @@ class AptInstaller(object):
                 else:
                     name = fields[0].decode('ascii')
                     status = PKG_NOT_INSTALLED
-                pkg, _s = pkgs_dict[name]
+                pkg, _ = pkgs_dict[name]
                 pkgs_dict[name] = pkg, status
         finally:
             p.wait()
@@ -166,19 +186,21 @@ class AptInstaller(object):
 
 
 def select_installer(pack, runs, target_distribution=THIS_DISTRIBUTION):
+    """Selects the right package installer for a Linux distribution.
+    """
     orig_distribution = runs[0]['distribution'][0].lower()
 
     # Checks that the distributions match
     if (set([orig_distribution, target_distribution]) ==
             set(['ubuntu', 'debian'])):
         # Packages are more or less the same on Debian and Ubuntu
-        logging.warning("Installing on %s but pack was generated on %s" % (
+        logging.warning("Installing on %s but pack was generated on %s",
                         target_distribution.capitalize(),
-                        orig_distribution.capitalize()))
+                        orig_distribution.capitalize())
     elif orig_distribution != target_distribution:
-        logging.error("Installing on %s but pack was generated on %s" % (
+        logging.error("Installing on %s but pack was generated on %s",
                       target_distribution.capitalize(),
-                      orig_distribution.capitalize()))
+                      orig_distribution.capitalize())
         sys.exit(1)
 
     # Selects installation method
@@ -189,7 +211,7 @@ def select_installer(pack, runs, target_distribution=THIS_DISTRIBUTION):
         installer = AptInstaller('apt-get')
     else:
         logging.critical("Your current distribution, \"%s\", is not "
-                         "supported" %
+                         "supported",
                          (target_distribution or "(unknown)").capitalize())
         sys.exit(1)
 
@@ -197,10 +219,14 @@ def select_installer(pack, runs, target_distribution=THIS_DISTRIBUTION):
 
 
 def busybox_url(arch):
+    """Gets the correct URL for the busybox binary given the architecture.
+    """
     return 'http://www.busybox.net/downloads/binaries/latest/busybox-%s' % arch
 
 
 def join_root(root, path):
+    """Prepends `root` to the absolute path `path`.
+    """
     p_root, p_loc = path.split_root()
     assert p_root == b'/'
     return root / p_loc
@@ -224,7 +250,10 @@ class FileUploader(object):
                 if len(runs) > 1:
                     print("  Run %d:" % i)
                 for input_name in run['input_files']:
-                    assigned = self.input_files.get(input_name) or "(original)"
+                    if self.input_files.get(input_name) is not None:
+                        assigned = PosixPath(self.input_files[input_name])
+                    else:
+                        assigned = "(original)"
                     print("    %s: %s" % (input_name, assigned))
             return
 
@@ -240,7 +269,7 @@ class FileUploader(object):
             for filespec in files:
                 filespec_split = filespec.rsplit(':', 1)
                 if len(filespec_split) != 2:
-                    logging.critical("Invalid file specification: %r" %
+                    logging.critical("Invalid file specification: %r",
                                      filespec)
                     sys.exit(1)
                 local_path, input_name = filespec_split
@@ -248,14 +277,14 @@ class FileUploader(object):
                 try:
                     input_path = PosixPath(all_input_files[input_name])
                 except KeyError:
-                    logging.critical("Invalid input file: %r" % input_name)
+                    logging.critical("Invalid input file: %r", input_name)
                     sys.exit(1)
 
                 temp = None
 
                 if not local_path:
                     # Restore original file from pack
-                    logging.debug("Restoring input file %s" % input_path)
+                    logging.debug("Restoring input file %s", input_path)
                     fd, temp = Path.tempfile(prefix='reprozip_input_')
                     os.close(fd)
                     local_path = self.extract_original_input(input_name,
@@ -263,10 +292,10 @@ class FileUploader(object):
                                                              temp)
                 else:
                     local_path = Path(local_path)
-                    logging.debug("Uploading file %s to %s" % (local_path,
-                                                               input_path))
+                    logging.debug("Uploading file %s to %s",
+                                  local_path, input_path)
                     if not local_path.exists():
-                        logging.critical("Local file %s doesn't exist" %
+                        logging.critical("Local file %s doesn't exist",
                                          local_path)
                         sys.exit(1)
 
@@ -274,7 +303,7 @@ class FileUploader(object):
 
                 if temp is not None:
                     temp.remove()
-                    self.input_files[input_name] = None
+                    del self.input_files[input_name]
                 else:
                     self.input_files[input_name] = local_path.absolute().path
         finally:
@@ -306,6 +335,8 @@ class FileUploader(object):
 
 
 class FileDownloader(object):
+    """Common logic for 'download' commands.
+    """
     def __init__(self, target, files):
         self.target = target
         self.run(files)
@@ -335,7 +366,7 @@ class FileDownloader(object):
             for filespec in files:
                 filespec_split = filespec.split(':', 1)
                 if len(filespec_split) != 2:
-                    logging.critical("Invalid file specification: %r" %
+                    logging.critical("Invalid file specification: %r",
                                      filespec)
                     sys.exit(1)
                 output_name, local_path = filespec_split
@@ -343,10 +374,10 @@ class FileDownloader(object):
                 try:
                     remote_path = PosixPath(all_output_files[output_name])
                 except KeyError:
-                    logging.critical("Invalid output file: %r" % output_name)
+                    logging.critical("Invalid output file: %r", output_name)
                     sys.exit(1)
 
-                logging.debug("Downloading file %s" % remote_path)
+                logging.debug("Downloading file %s", remote_path)
                 if not local_path:
                     self.download_and_print(remote_path)
                 else:
@@ -373,11 +404,11 @@ class FileDownloader(object):
         with temp.open('rb') as fp:
             chunk = fp.read(1024)
             if chunk:
-                sys.stdout.write(chunk)
+                sys.stdout.buffer.write(chunk)
             while len(chunk) == 1024:
                 chunk = fp.read(1024)
                 if chunk:
-                    sys.stdout.write(chunk)
+                    sys.stdout.buffer.write(chunk)
         temp.remove()
 
     def download(self, remote_path, local_path):
@@ -388,10 +419,15 @@ class FileDownloader(object):
 
 
 def get_runs(runs, selected_run, cmdline):
+    """Selects which run(s) to execute based on parts of the command-line.
+
+    Will return an iterable of run numbers. Might also fail loudly or exit
+    after printing the original command-line.
+    """
     if selected_run is None and len(runs) == 1:
         selected_run = 0
 
-    # --cmdline without arguments: display the original command line
+    # --cmdline without arguments: display the original command-line
     if cmdline == []:
         if selected_run is None:
             logging.critical("There are several runs in this pack -- you have "
