@@ -27,6 +27,7 @@ import sys
 import tarfile
 
 from reprounzip.common import load_config as load_config_file
+from reprounzip import signals
 from reprounzip.unpackers.common import THIS_DISTRIBUTION, PKG_NOT_INSTALLED, \
     COMPAT_OK, COMPAT_NO, target_must_exist, shell_escape, load_config, \
     select_installer, busybox_url, join_root, FileUploader, FileDownloader, \
@@ -72,10 +73,10 @@ def installpkgs(args):
             req = pkg.version
             real = pkgs[pkg.name][1]
             if real == PKG_NOT_INSTALLED:
-                logging.warning("package %s was not installed" % pkg.name)
+                logging.warning("package %s was not installed", pkg.name)
             else:
                 logging.warning("version %s of %s was installed, instead of "
-                                "%s" % (real, pkg.name, req))
+                                "%s", real, pkg.name, req)
         if r != 0:
             sys.exit(r)
 
@@ -84,7 +85,7 @@ def write_dict(filename, dct, type_):
     to_write = {'unpacker': type_}
     to_write.update(dct)
     with filename.open('wb') as fp:
-        pickle.dump(to_write, fp, pickle.HIGHEST_PROTOCOL)
+        pickle.dump(to_write, fp, 2)
 
 
 def read_dict(filename, type_):
@@ -118,6 +119,8 @@ def directory_create(args):
         logging.critical("Not unpacking on POSIX system")
         sys.exit(1)
 
+    signals.pre_setup(target=target, pack=pack)
+
     # Unpacks configuration file
     tar = tarfile.open(str(pack), 'r:*')
     member = tar.getmember('METADATA/config.yml')
@@ -141,8 +144,8 @@ def directory_create(args):
             if not f.exists():
                 logging.error(
                         "Missing file %s (from package %s that wasn't packed) "
-                        "on host, experiment will probably miss it." % (
-                            f, pkg.name))
+                        "on host, experiment will probably miss it.",
+                        f, pkg.name)
                 missing_files = True
     if missing_files:
         logging.error(
@@ -193,6 +196,8 @@ def directory_create(args):
     # Meta-data for reprounzip
     write_dict(target / '.reprounzip', {}, 'directory')
 
+    signals.post_setup(target=target)
+
 
 @target_must_exist
 def directory_run(args):
@@ -246,7 +251,7 @@ def directory_run(args):
                     for d in path
                     if d.root == '/']
         # Rebuild string
-        path = ':'.join(unicode_(d) for d in (dir_path + path))
+        path = ':'.join(unicode_(d) for d in dir_path + path)
         cmd += 'PATH=%s ' % shell_escape(path)
 
         # FIXME : Use exec -a or something if binary != argv[0]
@@ -258,7 +263,10 @@ def directory_run(args):
         cmds.append(cmd)
     cmds = ' && '.join(cmds)
 
-    subprocess.check_call(cmds, shell=True)
+    signals.pre_run(target=target)
+    retcode = subprocess.call(cmds, shell=True)
+    sys.stderr.write("\n*** Command finished, status: %d\n" % retcode)
+    signals.post_run(target=target, retcode=retcode)
 
 
 @target_must_exist
@@ -268,11 +276,15 @@ def directory_destroy(args):
     target = Path(args.target[0])
     read_dict(target / '.reprounzip', 'directory')
 
-    logging.info("Removing directory %s..." % target)
+    logging.info("Removing directory %s...", target)
+    signals.pre_destroy(target=target)
     rmtree_fixed(target)
+    signals.post_destroy(target=target)
 
 
 def should_restore_owner(param):
+    """Computes whether to restore original files' owners.
+    """
     if os.getuid() != 0:
         if param is True:
             # Restoring the owner was explicitely requested
@@ -300,6 +312,8 @@ def should_restore_owner(param):
 
 
 def should_mount_magic_dirs(param):
+    """Computes whether to mount directories inside the chroot.
+    """
     if os.getuid() != 0:
         if param is True:
             # Restoring the owner was explicitely requested
@@ -347,6 +361,8 @@ def chroot_create(args):
         logging.critical("Not unpacking on POSIX system")
         sys.exit(1)
 
+    signals.pre_setup(target=target, pack=pack)
+
     # We can only restore owner/group of files if running as root
     restore_owner = should_restore_owner(args.restore_owner)
 
@@ -368,16 +384,17 @@ def chroot_create(args):
     if packages_not_packed:
         logging.warning("According to configuration, some files were left out "
                         "because they belong to the following packages:%s"
-                        "\nWill copy files from HOST SYSTEM" % ''.join(
-                            '\n    %s' % pkg for pkg in packages_not_packed))
+                        "\nWill copy files from HOST SYSTEM",
+                        ''.join('\n    %s' % pkg
+                                for pkg in packages_not_packed))
         for pkg in packages_not_packed:
             for f in pkg.files:
                 f = Path(f.path)
                 if not f.exists():
                     logging.error(
                             "Missing file %s (from package %s) on host, "
-                            "experiment will probably miss it" % (
-                                f, pkg.name))
+                            "experiment will probably miss it",
+                            f, pkg.name)
                 dest = join_root(root, f)
                 dest.parent.mkdir(parents=True)
                 if f.is_link():
@@ -436,6 +453,8 @@ def chroot_create(args):
     # Meta-data for reprounzip
     write_dict(target / '.reprounzip', {}, 'chroot')
 
+    signals.post_setup(target=target)
+
 
 @target_must_exist
 def chroot_mount(args):
@@ -447,7 +466,7 @@ def chroot_mount(args):
     for m in ('/dev', '/proc'):
         d = join_root(target / 'root', Path(m))
         d.mkdir(parents=True)
-        logging.info("Mounting %s on %s..." % (m, d))
+        logging.info("Mounting %s on %s...", m, d)
         subprocess.check_call(['mount', '--bind', m, str(d)])
 
     write_dict(target / '.reprounzip', {'mounted': True}, 'chroot')
@@ -495,7 +514,10 @@ def chroot_run(args):
         cmds.append(cmd)
     cmds = ' && '.join(cmds)
 
-    subprocess.check_call(cmds, shell=True)
+    signals.pre_run(target=target)
+    retcode = subprocess.call(cmds, shell=True)
+    sys.stderr.write("\n*** Command finished, status: %d\n" % retcode)
+    signals.post_run(target=target, retcode=retcode)
 
 
 @target_must_exist
@@ -516,7 +538,7 @@ def chroot_destroy_unmount(args):
     for m in ('/dev', '/proc'):
         d = join_root(target / 'root', Path(m))
         if d.exists():
-            logging.info("Unmounting %s..." % d)
+            logging.info("Unmounting %s...", d)
             subprocess.check_call(['umount', str(d)])
 
 
@@ -531,8 +553,10 @@ def chroot_destroy_dir(args):
         logging.critical("Magic directories might still be mounted")
         sys.exit(1)
 
-    logging.info("Removing directory %s..." % target)
+    logging.info("Removing directory %s...", target)
+    signals.pre_destroy(target=target)
     rmtree_fixed(target)
+    signals.post_destroy(target=target)
 
 
 @target_must_exist
@@ -546,11 +570,13 @@ def chroot_destroy(args):
         for m in ('/dev', '/proc'):
             d = join_root(target / 'root', Path(m))
             if d.exists():
-                logging.info("Unmounting %s..." % d)
+                logging.info("Unmounting %s...", d)
                 subprocess.check_call(['umount', str(d)])
 
-    logging.info("Removing directory %s..." % target)
+    logging.info("Removing directory %s...", target)
+    signals.pre_destroy(target=target)
     rmtree_fixed(target)
+    signals.post_destroy(target=target)
 
 
 class LocalUploader(FileUploader):
@@ -615,11 +641,11 @@ class LocalDownloader(FileDownloader):
         with remote_path.open('rb') as fp:
             chunk = fp.read(1024)
             if chunk:
-                sys.stdout.write(chunk)
+                sys.stdout.buffer.write(chunk)
             while len(chunk) == 1024:
                 chunk = fp.read(1024)
                 if chunk:
-                    sys.stdout.write(chunk)
+                    sys.stdout.buffer.write(chunk)
 
     def download(self, remote_path, local_path):
         remote_path = join_root(self.root, remote_path)
@@ -751,6 +777,8 @@ def setup_directory(parser):
 
 
 def chroot_setup(args):
+    """Does both create and mount depending on --bind-magic-dirs.
+    """
     chroot_create(args)
     if should_mount_magic_dirs(args.bind_magic_dirs):
         chroot_mount(args)
