@@ -38,6 +38,8 @@ C_FORKEXEC = 3  # A fork then an exec, folded as one because all_forks==False
 
 
 class Process(CommonEqualityMixin):
+    """Structure representing a process in the experiment.
+    """
     def __init__(self, pid, parent, timestamp, acted, binary, created):
         self.pid = pid
         self.parent = parent
@@ -276,7 +278,7 @@ def graph(args):
                 tar.extract('METADATA/config.yml', path=str(tmp))
                 tar.extract('METADATA/trace.sqlite3', path=str(tmp))
             except KeyError as e:
-                logging.critical("Error extracting from pack: %s" % e.args[0])
+                logging.critical("Error extracting from pack: %s", e.args[0])
             generate(Path(args.target[0]),
                      tmp / 'METADATA',
                      args.all_forks)
